<?php

class IMAGEData implements Iterator {
 var $period, $multiplier;
 var $from, $to;
 var $resample, $nextsample;
 var $pos;
 
 var $items;

 var $start;
 
 const DEFAULT_START = "January 1, 2000";
   
 public function __construct(TESTReader &$reader, OPTIONS &$opts, &$items, INTERVAL &$ivl, $resample) {
    $period = $opts->Get('period');

    $this->multiplier = $reader->GetFractionalMultiplier($period);
    $this->period = $this->multiplier * $period;
    $this->resample = $this->multiplier * $resample;
    
    $from = $this->multiplier * $ivl->GetWindowStart();
    $ifrom = ceil($from);
    $rem = $ifrom % $this->period;
    if ($rem) $this->from = $ifrom + ($this->period - $rem);
    else $this->from = $ifrom;

    $this->to = $this->multiplier * $ivl->GetWindowEnd();
    
    $limit = $ivl->GetItemLimit();
    if ($limit) {
	if ($limit > 0) {
	    $to = $this->from + $this->period * $limit;
	    if ($to < $this->to) $this->to = $to;
	} else {
	    $from = $this->to + $this->period * $limit;
	    if ($from > $this->from) $this->from = $from;
	}
    }
    
    $this->items = &$items;

    $limit = $opts->GetDateLimit(TESTData::DEFAULT_START, time());
    $this->start = $limit[0]; 
 }
 
 function doResample() {
    for ($next = $this->pos + $this->period;(($next < $this->to)&&($next < $this->nextsample));$next += $this->period)
	$this->pos = $next;

    $this->nextsample += $this->resample;
    if ($this->nextsample < $this->pos) {
	$add = ceil(($this->pos - $this->nextsample) / $this->resample);
	$this->nextsample += $add * $this->resample;
    }
 }
 
 function rewind() {
    $this->pos = $this->from;

    if ($this->resample) {
	$this->nextsample = $this->resample*ceil($this->pos / $this->resample);
	$this->doResample();
    }
 }

 function current() {
    $x = ($this->pos - $this->start) / $this->multiplier;
    $period = $this->period / $this->multiplier;

    $max = getrandmax();
    
    $values = array();
    foreach ($this->items as &$item) {
	$y = 0;

	if (eval($item['func'] . ";") === false)
	    throw new ADEIException(translate("Invalid function(%s) is supplied to TESTReader", $item['func']));

	if ($item['noise']) $y += 2*$item['noise']*((rand() - $max/2)/$max);

	array_push($values, $y);
    }
    return $values;
 }

 function key() {
    $res = $this->pos / $this->multiplier;
    if (is_float($res)) return sprintf("%.8f", $res);
    return $res;
 }

 function next() {
    $this->pos += $this->period;

    if ($this->resample) $this->doResample();
 }

 function valid() {
    return ($this->pos < $this->to);
 }

}



class IMAGEReader extends READER {
 var $cache;

 var $items;
 var $groups;
 
 function __construct(&$props) {
    parent::__construct($props);

    $this->items = array(
	
    );
    

    if ($this->dbname) {
	$list = $this->opts->ListConfiguredGroups();
    
        if ($list) {
	    $this->groups = array();
	
	    foreach ($list as $gid) {
		$this->groups[$gid] = array();
	    }
	} else {
	    $this->groups = array(
		'default' => array(
		    'name' => _("Default Group")
		)
	    );
	}
    }

/*
    if (sizeof($list) > 1) {
	array_push($this->groups, array(
	    'id' => "-1",
	    'name' => _("Combined Group"),
	    'complex' => true
	);
    }
*/
 }
 
 function GetGroupInfo(LOGGROUP $grp = NULL, $flags = 0) {
    return NULL;
 }
 
 function GetItemList(LOGGROUP $grp = NULL, MASK $mask = NULL, $flags = 0) {
    return NULL;
 }

 function GetRawData(LOGGROUP $grp = NULL, $from = 0, $to = 0, DATAFilter $filter = NULL, &$filter_data = NULL) {
    return new TESTData($this, $this->req->GetGroupOptions($grp), $items, $ivl, $resample);
 }

 function HaveData(LOGGROUP $grp = NULL, $from = 0, $to = 0) {
    $grp = $this->CheckGroup($grp);

    $ivl = $this->CreateInterval($grp);
    $ivl->Limit($from, $to);
    
    $period = $this->req->GetGroupOption('period', $grp);

    $from = $ivl->GetWindowStart();
    $to = $ivl->GetWindowEnd();
    
    if (($from - $to) > 2 * $period) return true;
    else {
	$multiplier = $this->GetFractionalMultiplier($period);
	$period *= $multiplier; 
	
	$ifrom = ceil($multiplier * $from);
	$rem = $ifrom % $period;
	if ($rem) $curfrom = $ifrom + ($period - $rem);
	else $curfrom = $ifrom;
	
	if ($curfrom < $multiplier * $to) return true;
    }
    
    return false;
 }
}

?>