<?xml version="1.0" encoding="utf-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">
<xsl:output encoding="utf-8"/>

<xsl:template match="streams">
    <xsl:for-each select="type">
        <xsl:variable name="title">
            <xsl:choose>
                <xsl:when test="@name=uw">UW</xsl:when>
                <xsl:when test="@name='ipe3_energy'">IPE3 Energy</xsl:when>
                <xsl:when test="@name='ipe3_waveform'">IPE3 Waveform</xsl:when>
                <xsl:when test="@name='ipe3_histogram'">IPE3 Histogram</xsl:when>
                <xsl:when test="@name='ipe4_energy'">IPE4 Energy</xsl:when>
                <xsl:when test="@name='ipe4_waveform'">IPE4 Waveform</xsl:when>
                <xsl:when test="@name='ipe4_histogram'">IPE4 Histogram</xsl:when>
            </xsl:choose>
        </xsl:variable>
        <xsl:element name="img">
            <xsl:attribute name="src">
                <xsl:text>setups/katrin/images/</xsl:text>
                <xsl:value-of select="@name" />
                <xsl:text>.png</xsl:text>
            </xsl:attribute>
            <xsl:attribute name="height">16</xsl:attribute>
            <xsl:attribute name="alt">
                <xsl:value-of select="$title" />
            </xsl:attribute>
            <xsl:attribute name="title">
                <xsl:value-of select="$title" />
            </xsl:attribute>
            <xsl:attribute name="style">
                <xsl:text>margin: 0 5px; vertical-align: middle</xsl:text>
            </xsl:attribute>
        </xsl:element>
    </xsl:for-each>
</xsl:template>

<!-- Helper template for page links in last runs view -->
<xsl:template name="page_link">
    <xsl:param name="page" />
    <xsl:param name="current" />

    <xsl:variable name="text">
        <xsl:text>[</xsl:text>
        <xsl:value-of select="$page" />
        <xsl:text>]</xsl:text>
    </xsl:variable>

    <xsl:if test="$page=$current">
        <xsl:value-of select="$text" />
    </xsl:if>
    <xsl:if test="not($page=$current)">
        <xsl:element name="a">
            <xsl:attribute name="href">
                <xsl:text>javascript:katrin.SetCustomProperties('kdb_page=</xsl:text>
                <xsl:value-of select="$page" />
                <xsl:text>')</xsl:text>
            </xsl:attribute>
            <xsl:value-of select="$text" />
        </xsl:element>
    </xsl:if>
</xsl:template>

<xsl:template name="navigation">
    <xsl:text>Page: </xsl:text>
    <xsl:call-template name="page_link">
        <xsl:with-param name="page" select="1" />
        <xsl:with-param name="current" select="@page" />
    </xsl:call-template>
    <xsl:if test="number(@page)&gt;3">
        <xsl:text> ... </xsl:text>
    </xsl:if>
    <xsl:if test="number(@page)&gt;2">
        <xsl:call-template name="page_link">
            <xsl:with-param name="page" select="number(@page)-1" />
            <xsl:with-param name="current" select="@page" />
        </xsl:call-template>
    </xsl:if>
    <xsl:if test="number(@page)&gt;1">
        <xsl:call-template name="page_link">
            <xsl:with-param name="page" select="@page" />
            <xsl:with-param name="current" select="@page" />
        </xsl:call-template>
    </xsl:if>
    <xsl:if test="number(@page)&lt;number(@pages)">
        <xsl:call-template name="page_link">
            <xsl:with-param name="page" select="number(@page)+1" />
            <xsl:with-param name="current" select="@page" />
        </xsl:call-template>
    </xsl:if>
    <xsl:if test="number(@page)&lt;number(@pages)-2">
        <xsl:text> ... </xsl:text>
    </xsl:if>
    <xsl:if test="number(@page)&lt;number(@pages)-1">
        <xsl:call-template name="page_link">
            <xsl:with-param name="page" select="@pages" />
            <xsl:with-param name="current" select="@page" />
        </xsl:call-template>
    </xsl:if>
</xsl:template>

<xsl:template match="status">
    <xsl:variable name="color">
        <xsl:choose>
            <xsl:when test=". = 'Ok'">
                <xsl:text>#008000</xsl:text>
            </xsl:when>
            <xsl:when test=". = 'Error'">
                <xsl:text>#800000</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:text>#000000</xsl:text>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:variable>

    <xsl:element name="span">
        <xsl:attribute name="style">
            <xsl:text>font-weight: bold; color: </xsl:text>
            <xsl:value-of select="$color" />
        </xsl:attribute>
        <xsl:value-of select="." />
    </xsl:element>
</xsl:template>

<!-- Template for runs list in last runs view -->
<xsl:template match="runs">
    <table width="100%">
        <tr>
            <td style="text-align:left">
                <xsl:call-template name="navigation" />
            </td>
            <td style="text-align:right">
                <xsl:if test="string-length(@filter) > 0">
                    <a href="javascript:adei.OpenControl('filter')">
                        <xsl:text>Filter</xsl:text>
                    </a>
                    <xsl:text>: </xsl:text>
                    <span class="search">
                        <xsl:value-of select="@filter" />
                    </span>
                    <br />
                </xsl:if>
                <xsl:if test="string-length(@search) > 0">
                    <xsl:text>Search: "</xsl:text>
                    <span class="search">
                        <xsl:value-of select="@search" />
                    </span>
                    <xsl:text>" (</xsl:text>
                    <a href="javascript:katrin.SetCustomProperties('kdb_search=&amp;kdb_parameters=&amp;kdb_date=')">
                        <xsl:text>Clear</xsl:text>
                    </a>
                    <xsl:text>)</xsl:text>
                    <br />
                </xsl:if>
                <xsl:if test="string-length(@window) > 0">
                    <xsl:text>Selected time window (</xsl:text>
                    <a href="javascript:katrin.SetCustomProperties('kdb_window=&amp;kdb_date=')">
                        <xsl:text>Clear</xsl:text>
                    </a>
                    <xsl:text>)</xsl:text>
                </xsl:if>
            </td>
        </tr>
    </table>
    <xsl:if test="count(run) > 0">
        <table class="border" border="1" cellspacing="0" cellpadding="5" width="100%">
            <tr class="border">
                <th class="border overview">Identifier</th>
                <th class="border overview">Start (UTC)</th>
                <th class="border overview">Duration</th>
                <th class="border overview">Subruns</th>
                <th class="border overview">Data</th>
                <th class="border overview">Status</th>
                <th class="border" style="width:100%">Comment</th>
            </tr>
            <xsl:for-each select="run">
                <tr>
                    <td class="border overview">
                        <xsl:element name="a">
                            <xsl:attribute name="href">
                                <xsl:text>javascript:katrin.SetCustomProperties('kdb_run=</xsl:text>
                                <xsl:value-of select="identifier" />
                                <xsl:text>&amp;kdb_subruns=')</xsl:text>
                            </xsl:attribute>
                            <xsl:value-of select="identifier" />
                        </xsl:element>
                    </td>
                    <td class="border overview date">
                        <xsl:value-of select="start" />
                    </td>
                    <td class="border overview">
                        <xsl:value-of select="duration" />
                    </td>
                    <td class="border overview">
                        <xsl:value-of select="subruns" />
                    </td>
                    <td class="border overview">
                        <xsl:apply-templates select="streams" />
                    </td>
                    <td class="border overview">
                        <xsl:apply-templates select="status" />
                    </td>
                    <td class="border overview" style="text-align: left">
                        <xsl:value-of select="comment" />
                    </td>
                </tr>
            </xsl:for-each>
        </table>
    </xsl:if>
    <xsl:if test="count(run) = 0">
        <div>No runs found</div>
    </xsl:if>
    <table width="100%">
        <tr>
            <td style="text-align:left">
                <xsl:call-template name="navigation" />
            </td>
            <td style="text-align:right">
            </td>
        </tr>
    </table>
</xsl:template>

<xsl:template match="loading">
    <script type="text/javascript">
        adei.OpenControl('filter');
        filter.SetSearch('<xsl:value-of select="@search"/>', '<xsl:value-of select="@parameters"/>', '<xsl:value-of select="@window"/>');
    </script>
</xsl:template>

<xsl:template match="error">
    <h3>Invalid filter</h3>
    <p><xsl:value-of select="." /></p>
</xsl:template>

<xsl:template match="help">
    <script type="text/javascript">
        filter.FocusParameters();
    </script>
    <xsl:apply-templates select="error" />
    <h3>Filter Description</h3>
    <p>The run list can be filtered by defining a filter with predefined
        parameters.</p>
    <p>The general filter format is:</p>
    <p class="code">parameter OP value[, parameter OP value, ...]</p>
    <ul>
        <li>
            <span class="variable">OP</span> can be one of &lt;, &lt;=, =,
            &gt;= or &gt;
        </li>
        <li>
            <span class="variable">value</span> is a numeric value, scientific
            notation is allowed
        </li>
        <li>
            <span class="variable">parameter</span> is a predefined parameter
            name. Currently the following parameters are defined:
            <div style="padding=5px">
                <table class="border" border="1">
                    <tr class="border">
                        <th class="border">Parameter</th>
                        <th class="border">Description</th>
                    </tr>
                    <xsl:for-each select="parameter">
                        <tr class="border">
                            <td class="border">
                                <xsl:value-of select="name" />
                            </td>
                            <td class="border">
                                <xsl:value-of select="description" />
                            </td>
                        </tr>
                    </xsl:for-each>
                </table>
            </div>
        </li>
    </ul>
    <p>If more than one filter is specified they are combined using an AND
    operation.</p>
</xsl:template>

<xsl:template match="/result">
    <div style="margin: 10px">
        <xsl:choose>
            <xsl:when test="Error">
                <h2>Error</h2>
                <xsl:value-of select="Error" />
            </xsl:when>
            <xsl:when test="runs">
                <xsl:apply-templates select="runs" />
            </xsl:when>
            <xsl:when test="loading">
                <xsl:apply-templates select="loading" />
            </xsl:when>
            <xsl:when test="help">
                <xsl:apply-templates select="help" />
            </xsl:when>
        </xsl:choose>
    </div>
</xsl:template>
</xsl:stylesheet>
